{

=pod

=head1 NAME

SCNS::XMLRPC

=head1 SUMMARY 

 my $service = SCNS::XMLRPC->new(uri=>"http://betty.userland.com/RPC2",class=>"examples");
 print $service->getStateName(40);

=head1 DESCRIPTION

=cut

package SCNS::XMLRPC;
use strict;

use Carp;
use XMLRPC::Lite;

use vars qw ($AUTOLOAD);

$SCNS::XMLRPC::VERSION = 0.1;

use constant LASTERROR => join("::",__PACKAGE__,"__lasterror");

sub new {
    my $pkg = shift;
    
    my $self = {};
    bless $self,$pkg;

    if (! $self->init(@_)) {
	return undef;
    }

    return $self;
}

sub init {
    my $self = shift;
    my $uri  = shift;

    $self->{"_client"} = XMLRPC::Lite->new();
    $self->{"_client"}->proxy($uri);
    $self->{"_client"}->on_fault(\&_fault);

    return 1;
}

sub class {
    my $self  = shift;
    my $class = shift;

    if (defined($class)) {
	$self->{"_class"} = $class;
    }

    return $class;
}

sub debug {
    my $self = shift;
    my $bool = shift;
    my $fh   = shift;

    if ($bool) {
	if (! defined($fh)) { $fh = *STDERR; }
	$self->{"_client"}->on_debug(sub { print $fh @_; });
    } 

    else {
	$self->{"_client"}->on_debug(sub { return; });
    }

    return 1;
}

sub last_error {
    my $self = shift;
    return $self->{"_client"}->{LASTERROR};
}

sub _call {
    my $self   = shift;
    my $method = shift;
    my @args   = @_;

    my $response = $self->{"_client"}->call($method,@args);
    if (! UNIVERSAL::can($response,"result")) { return undef; }

    return $response->result();
}

sub _fault {
    my $client = shift;
    my $error  = shift;
    $client->{LASTERROR} = $error->faultstring();
}

sub DESTROY {
    my $self = shift;
    return 1;
}

sub AUTOLOAD {
    my $self = shift;

    $AUTOLOAD =~ /.*::(.*)$/;
    my $method = $1;

    if (my $class = $self->{'_class'}) {
	$method = join(".",$class,$method);
    }

    return $self->_call($method,@_);
}

=pod

=head1 VERSION 

0.1

=head1 DATE

$Date: 2002/02/08 15:00:58 $

=head1 AUTHOR

Aaron Straup Cope

=head1 SEE ALSO

L<XMLRPC::Lite>

=head1 TO DO

=over 4

=item

Finished installing (debugging) AUTOLOAD subs

=item

POD

=back

=head1 CHANGES

=head2 0.1

=over 4

=item

Initial revision

=back

=head1 LICENSE

Copyright (c) 2002 Aaron Straup Cope.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
